==> mutual_credit-5.0.x-dev.tar.gz: mutual_credit/src/Entity/ViewBuilder/TransactionViewBuilder.php <==
<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\TransactionOperations;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for transactions.
 */
class TransactionViewBuilder extends EntityViewBuilder {

  protected $settings;
  protected $routeMatch;
  protected $token;

  /**
   * @param $entity_type
   * @param $entity_repository
   * @param $language_manager
   * @param $theme_registry
   * @param $entity_display_repository
   * @param ConfigFactory $config_factory
   * @param CurrentRouteMatch $route_match
   * @param Token $token
   */
  public function __construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository, ConfigFactory $config_factory, CurrentRouteMatch $route_match, Token $token) {
    parent::__construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository);
    $this->settings = $config_factory->get('mcapi.settings');
    $this->routeMatch = $route_match;
    $this->token = $token;
  }

  /**
   * {@inheritDoc}
   *
   * @todo update with entity_type.manager when core interface changes
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.repository'),
      $container->get('language_manager'),
      $container->get('theme.registry'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('current_route_match'),
      $container->get('token')
    );
  }

  /**
   * {@inheritDoc}
   */
  protected function getBuildDefaults(EntityInterface $entity, $view_mode) {
    $build = parent::getBuildDefaults($entity, $view_mode);
    switch ($view_mode) {
      case 'full':
        $build['#links'] = [];
        if ($this->routeMatch->getRouteName() == 'entity.mcapi_transaction.canonical') {
          $build['#links'] = $this->linkList($entity, FALSE);
        }
        $build['#watermark'] = '';
        break;

      case 'sentence':
        $template = $this->settings->get('sentence_template');
        if ($entity->type->target_id == 'inherit' or $entity->type->target_id == 'mass' && empty($entity->children->getReferencedEntities)) {
          // only show links for the parent transaction
        }
        else {
          $template .= ' [xaction:links]';
        }
        $build = [
          '#prefix' => '<div class = "transaction-sentence">',
          '#markup' => \Drupal::token()->replace($template, ['xaction' => $entity]),
          '#suffix' => '</div>'
        ];
    }
    $build += [
      '#attributes' => [
        'class' => [
          'transaction',
          'type-' . $entity->type->target_id,
          'state-' . $entity->state->target_id,
          ($view_mode == 'full' ? 'certificate' : $view_mode)
        ]
      ],
      '#attached' => [
        'library' => ['mcapi/transaction'],
      ],
    ];
    // @todo we might need to use the post-render cache to get the links right instead of template_preprocess_mcapi_transaction
    return $build;
  }

  /**
   * Make a renderable array for transaction operations
   *
   * @param Transaction $transaction
   * @param bool $include_view_op
   *
   * @return array
   *   A renderable array
   */
  private function linkList(Transaction $transaction, $include_view_op = TRUE) : array {
    $renderable = [];
    if (!$transaction->isNew()) {
      return [
        '#lazy_builder' => [
          get_called_class() . '::renderLinks', [
            $transaction->serial->value,
            $include_view_op
          ]
        ]
      ];
    }
    return $renderable;
  }

  /**
   * @param int $serial
   * @param bool $include_view_op
   *
   * @todo Better cache management of the links.
   */
  public static function renderlinks(int $serial, $include_view_op = TRUE) : array {
    $output = [
      '#theme' => 'item_list',
      '#type' => 'ul',
      '#items' => [],
      '#attributes' => ['class' => ['transaction-operations']],
    ];
    $operations = TransactionOperations::get(
      Transaction::loadBySerial($serial),
      $include_view_op
    );
    foreach ($operations as $name => $op) {
      $dest_route = ($name == 'delete') ? 'entity.mcapi_transaction.collection' : '<current>';
      $op['url']->setOption(
        'query',
        ['destination' => Url::fromRoute($dest_route)->toString()]
      );
      $output['#items'][] = Link::fromTextAndUrl($op['title'], $op['url']);
    }
    return $output;
  }

  /**
   * {@inheritDoc}
   */
  public static function trustedCallbacks() {
    $callbacks = parent::trustedCallbacks();
    $callbacks[] = 'renderLinks';
    return $callbacks;
  }

}

==> mutual_credit-5.2.x-dev.tar.gz: mutual_credit/src/Entity/ViewBuilder/TransactionViewBuilder.php <==
<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\TransactionOperations;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for transactions.
 */
class TransactionViewBuilder extends EntityViewBuilder {

  protected $settings;
  protected $routeMatch;
  protected $token;

  /**
   * @param $entity_type
   * @param $entity_repository
   * @param $language_manager
   * @param $theme_registry
   * @param $entity_display_repository
   * @param ConfigFactory $config_factory
   * @param CurrentRouteMatch $route_match
   * @param Token $token
   */
  public function __construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository, ConfigFactory $config_factory, CurrentRouteMatch $route_match, Token $token) {
    parent::__construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository);
    $this->settings = $config_factory->get('mcapi.settings');
    $this->routeMatch = $route_match;
    $this->token = $token;
  }

  /**
   * {@inheritDoc}
   *
   * @todo update with entity_type.manager when core interface changes
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.repository'),
      $container->get('language_manager'),
      $container->get('theme.registry'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('current_route_match'),
      $container->get('token')
    );
  }

  /**
   * {@inheritDoc}
   */
  protected function getBuildDefaults(EntityInterface $entity, $view_mode) {
    $build = parent::getBuildDefaults($entity, $view_mode);
    switch ($view_mode) {
      case 'full':
        $build['#links'] = [];
        if ($this->routeMatch->getRouteName() == 'entity.mcapi_transaction.canonical') {
          $build['#links'] = $this->linkList($entity, FALSE);
        }
        $build['#watermark'] = '';
        break;

      case 'sentence':
        $template = $this->settings->get('sentence_template');
        // this doesn't cover remote credcom entries.
        if ($entity->type->target_id == 'inherit' or $entity->type->target_id == 'mass' && empty($entity->children->getReferencedEntities)) {
          // only show links for the parent transaction
        }
        else {
          //$template .= ' [xaction:links]';
        }
        $template = str_replace(['[xaction:', ']'], ['{{ ', ' }}'], $template);
        $build = [[
          '#type' => 'inline_template',
          '#template' => '<div class = "transaction-sentence">'. $template .'</div>',
          '#context' => [
            'payer' => $entity->payer->entity->label(),
            'payee' => $entity->payee->entity->label(),
            'description' => $entity->description->value,
            'quant' => $entity->worth->format(),
          ],
        ]];
    }
    $build += [
      '#attributes' => [
        'class' => [
          'transaction',
          'type-' . $entity->type->target_id,
          'state-' . $entity->state->target_id,
          ($view_mode == 'full' ? 'certificate' : $view_mode)
        ]
      ],
      '#attached' => [
        'library' => ['mcapi/transaction'],
      ],
    ];
    // @todo we might need to use the post-render cache to get the links right instead of template_preprocess_mcapi_transaction
    return $build;
  }

  /**
   * Make a renderable array for transaction operations
   *
   * @param Transaction $transaction
   * @param bool $include_view_op
   *
   * @return array
   *   A renderable array
   */
  private function linkList(Transaction $transaction, $include_view_op = TRUE) : array {
    $renderable = [];
    if (!$transaction->isNew()) {
      return [
        '#lazy_builder' => [
          get_called_class() . '::renderLinks', [
            $transaction->serial->value,
            $include_view_op
          ]
        ]
      ];
    }
    return $renderable;
  }

  /**
   * @param int $serial
   * @param bool $include_view_op
   *
   * @todo Better cache management of the links.
   */
  public static function renderlinks(int $serial, $include_view_op = TRUE) : array {
    $output = [
      '#theme' => 'item_list',
      '#type' => 'ul',
      '#items' => [],
      '#attributes' => ['class' => ['transaction-operations']],
    ];
    $operations = TransactionOperations::get(
      Transaction::loadBySerial($serial),
      $include_view_op
    );
    foreach ($operations as $name => $op) {
      $dest_route = ($name == 'delete') ? 'entity.mcapi_transaction.collection' : '<current>';
      $op['url']->setOption(
        'query',
        ['destination' => Url::fromRoute($dest_route)->toString()]
      );
      $output['#items'][] = Link::fromTextAndUrl($op['title'], $op['url']);
    }
    return $output;
  }

  /**
   * {@inheritDoc}
   */
  public static function trustedCallbacks() {
    $callbacks = parent::trustedCallbacks();
    $callbacks[] = 'renderLinks';
    return $callbacks;
  }

}
